// A stream experiment.

#include <limits>
#include <iterator>

namespace stream {

template<typename T>
class iota_class
{
public:
    explicit iota_class(T value = T()) : value_(value) { }
    operator bool() { return value_ < std::numeric_limits<T>::max(); }
    T operator()() { return value_++; }
private:
    T value_;
};

template<typename T>
iota_class<T> iota(T start = T())
{
    return iota_class<T>(start);
}

template <typename Str>
class take_class
{
public:
    take_class(size_t size, Str stream)
            : stream_(std::move(stream)), size_(size)
    { }

    size_t size() { return size_; }

    operator bool() { return size_ > 0; }

    auto operator()() {
        --size_;
        return stream_();
    }

private:
    Str stream_;
    size_t size_;
};

template <typename Str>
take_class<Str> take(size_t size, Str stream)
{
    return take_class<Str>(size, std::move(stream));
}

template<typename Str, typename OutIter>
void collect(Str stream, OutIter out)
{
    while (stream)
        *out++ = stream();
};

template<typename Result, typename Str>
Result collect(Str stream)
{
    Result result;
    collect(stream, std::back_inserter(result));
    return result;
};

} // end namespace stream
